@@ -1,5 +1,5 @@
 /* charset.c -- charset module.
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -353,7 +353,8 @@ load_charset (FILE *fp, MSymbol charset_name)
       char buf[256];
 
       ungetc (c, fp);
-      fgets (buf, 256, fp);
+      if (! fgets (buf, 256, fp))
+	break;
       if (c != '#')
 	{
 	  if (sscanf (buf, "0x%x-0x%x 0x%x", &code1, &code2, &c1) == 3)
@@ -672,7 +673,7 @@ mcharset__load_from_database ()
     return 0;
   MDEBUG_PUSH_TIME ();
   def_list = (MPlist *) mdatabase_load (mdb);
-  MDEBUG_PRINT_TIME ("CHARSET", (stderr, " to load data."));
+  MDEBUG_PRINT_TIME ("CHARSET", (mdebug__output, " to load data."));
   MDEBUG_POP_TIME ();
   if (! def_list)
     return -1;
@@ -698,7 +699,7 @@ mcharset__load_from_database ()
     }
 
   M17N_OBJECT_UNREF (def_list);
-  MDEBUG_PRINT_TIME ("CHARSET", (stderr, " to parse the loaded data."));
+  MDEBUG_PRINT_TIME ("CHARSET", (mdebug__output, " to parse the loaded data."));
   MDEBUG_POP_TIME ();
   return 0;
 }
@@ -950,7 +951,7 @@ CHARACTER-CODE = CODE-POINT - MIN-CODE + LOWEST-CHAR-CODE
 
 /***ja @brief ˥եΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Minherit  <tt>"unify"</tt> Ȥ̾ʸåȤ 
+    ܥ #Munify  <tt>"unify"</tt> Ȥ̾ʸåȤ 
     @b Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤʸåȤʸɤδ֤ѴޥåפλȤȥեåȤȤ߹碌ˤäƹԤ뤳Ȥ̣롣
     ޥåפ @b Mmapfile ѥ᡼ȤͿʤФʤʤ
     μγʸåȤˤϡʸФϢ³륳ɥڡ줾Ƥ롣
@@ -985,7 +986,7 @@ CHARACTER-CODE = PARENT-CODE (CODE-POINT) + SUBSET-OFFSET
 
     where, PARENT-CODE is a pseudo function that returns a character
     code of CODE-POINT in the parent charset, and SUBSET-OFFSET is a
-    value given by #Msubset_offset parameter.  */
+    value given by @b Msubset_offset parameter.  */
 
 /***ja @brief ֥åȷΥ᥽åɤ򼨤ܥ.
 
