@@ -1,5 +1,5 @@
 /* face.c -- face module.
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -1023,13 +1023,13 @@ MSymbol Mbackground;
     @brief Key of a face property specifying video mode.
 
     The variable #Mvideomode is used as a key of face property.  The
-    property value must be #Mnormal, #Mreverse, or #Mnil.
+    property value must be @b Mnormal, @b Mreverse, or #Mnil.
 
-    #Mnormal means that an M-text is drawn in normal video mode
+    @b Mnormal means that an M-text is drawn in normal video mode
     (i.e. the foreground is drawn by foreground color, the background
     is drawn by background color).
 
-    #Mreverse means that an M-text is drawn in reverse video mode
+    @b Mreverse means that an M-text is drawn in reverse video mode
     (i.e. the foreground is drawn by background color, the background
     is drawn by foreground color).
 
@@ -1038,12 +1038,12 @@ MSymbol Mbackground;
     @brief ӥǥ⡼ɤꤹ뤿ΥեץѥƥΥ.
 
     ѿ #Mvideomode ϥեץѥƥΥȤѤ롣ץѥƥͤϡ
-    #Mnormal, #Mreverse, #Mnil Τ줫ǤʤƤϤʤʤ
+    @b Mnormal, @b Mreverse, #Mnil Τ줫ǤʤƤϤʤʤ
 
-    #Mnormal ξϡM-text 
+    @b Mnormal ξϡM-text 
     ɸΥӥǥ⡼ɡʤʿǡطʤطʿǡˤɽ롣
 
-    #Mreverse ξϥСӥǥ⡼ɤǡʤطʿǡطʤʿǡɽ롣
+    @b Mreverse ξϥСӥǥ⡼ɤǡʤطʿǡطʤʿǡɽ롣
 
     #Mnil ξϥӥǥ⡼ɤϻꤵʤ
     */
@@ -1176,7 +1176,6 @@ MSymbol Mhook_arg;
 /***en @name Variables: Possible values of #Mvideomode property of face */
 /***ja @name ѿ  ե #Mvideomode ץѥƥβǽ */
 /*** @{ */
-/*=*/
 
 /***en
     See the documentation of the variable #Mvideomode.  */ 
@@ -1197,14 +1196,14 @@ MSymbol Mreverse;
     @brief Normal video face.
 
     The variable #mface_normal_video points to a face that has the
-    #Mvideomode property with value #Mnormal.  The other properties
+    #Mvideomode property with value @b Mnormal.  The other properties
     are not specified.  An M-text drawn with this face appear normal
     colors (i.e. the foreground is drawn by foreground color, and
     background is drawn by background color).  */
 /***ja
     @brief ɸӥǥե.
 
-    ѿ #mface_normal_video  #Mvideomode ץѥƥͤ #Mnormal 
+    ѿ #mface_normal_video  #Mvideomode ץѥƥͤ @b Mnormal 
     Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
     ΥեɽM-text 
     ɸο (ʤʤʿطʤطʿˤ롣  */
@@ -1215,7 +1214,7 @@ MFace *mface_normal_video;
     @brief Reverse video face.
 
     The variable #mface_reverse_video points to a face that has the
-    #Mvideomode property with value #Mreverse.  The other properties
+    #Mvideomode property with value @b Mreverse.  The other properties
     are not specified.  An M-text drawn with this face appear in
     reversed colors (i.e. the foreground is drawn by background
     color, and background is drawn by foreground color).  */
@@ -1223,7 +1222,7 @@ MFace *mface_normal_video;
     @brief Сӥǥե.
 
     ѿ #mface_reverse_video  #Mvideomode ץѥƥͤ 
-    #Mreverse Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    @b Mreverse Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
     ΥեɽM-text 
     ʿطʿؤä (ʤʤطʿطʤʿ롣  */
 
@@ -2052,9 +2051,10 @@ mface_update (MFrame *frame, MFace *face)
 /***en
     @brief Dump a face.
 
-    The mdebug_dump_face () function prints face $FACE in a human readable
-    way to the stderr.  $INDENT specifies how many columns to indent
-    the lines but the first one.
+    The mdebug_dump_face () function prints face $FACE in a human
+    readable way to the stderr or to what specified by the environment
+    variable MDEBUG_OUTPUT_FILE.  $INDENT specifies how many columns
+    to indent the lines but the first one.
 
     @return
     This function returns $FACE.  */
@@ -2062,8 +2062,9 @@ mface_update (MFrame *frame, MFace *face)
 /***ja
     @brief եפ.
 
-    ؿ mdebug_dump_face () ϥե $FACE  stderr 
-    ˿ʹ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
+    ؿ mdebug_dump_face () ϥե $FACE ɸ२顼Ϥ⤷
+    Ķѿ MDEBUG_DUMP_FONT ǻꤵ줿ե˿ʹ֤˲ɤʷǰ
+    롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
 
     @return
     δؿ $FACE ֤  */
@@ -2077,15 +2078,17 @@ mdebug_dump_face (MFace *face, int indent)
   memset (prefix, 32, indent);
   prefix[indent] = 0;
   mfont__set_spec_from_face (&spec, face);
-  fprintf (stderr, "(face font:\"");
+  fprintf (mdebug__output, "(face font:\"");
   mdebug_dump_font (&spec);
-  fprintf (stderr, "\"\n %s  fore:%s back:%s", prefix,
+  fprintf (mdebug__output, "\"\n %s  fore:%s back:%s", prefix,
 	   msymbol_name ((MSymbol) face->property[MFACE_FOREGROUND]),
 	   msymbol_name ((MSymbol) face->property[MFACE_BACKGROUND]));
   if (face->property[MFACE_FONTSET])
-    fprintf (stderr, " non-default-fontset");
-  fprintf (stderr, " hline:%s", face->property[MFACE_HLINE] ? "yes" : "no");
-  fprintf (stderr, " box:%s)", face->property[MFACE_BOX] ? "yes" : "no");
+    fprintf (mdebug__output, " non-default-fontset");
+  fprintf (mdebug__output, " hline:%s",
+	   face->property[MFACE_HLINE] ? "yes" : "no");
+  fprintf (mdebug__output, " box:%s)",
+	   face->property[MFACE_BOX] ? "yes" : "no");
   return face;
 }
 
