@@ -1,5 +1,5 @@
 /* m17n-gd.c -- implementation of the GUI API on GD Library.
-   Copyright (C) 2004, 2005, 2006
+   Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -775,7 +775,8 @@ gd_dump_region (MDrawRegion region)
   MDrawMetric rect;
 
   gd_region_to_rect (region, &rect);
-  fprintf (stderr, "(%d %d %d %d)\n", rect.x, rect.y, rect.width, rect.height);
+  fprintf (mdebug__output, "(%d %d %d %d)\n",
+	   rect.x, rect.y, rect.width, rect.height);
 }
 
 static MDeviceDriver gd_driver =
