@@ -1,5 +1,5 @@
 /* m17n-gui.c -- body of the GUI API.
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -125,9 +125,11 @@ register_device_library (MSymbol name, char *lib)
   MDeviceLibraryInterface *interface;
 
   MSTRUCT_CALLOC (interface, MERROR_WIN);
-  interface->library = malloc (strlen (lib) 
+  interface->library = malloc (strlen (M17N_MODULE_DIR) + 1
+			       + strlen (lib) 
 			       + strlen (DLOPEN_SHLIB_EXT) + 1);
-  sprintf (interface->library, "%s%s", lib, DLOPEN_SHLIB_EXT);
+  sprintf (interface->library, "%s/%s%s", M17N_MODULE_DIR, lib,
+	   DLOPEN_SHLIB_EXT);
   if (! device_library_list)
     device_library_list = mplist ();
   mplist_add (device_library_list, name, interface);
@@ -270,19 +272,20 @@ m17n_init_win (void)
   MDEBUG_PUSH_TIME ();
   if (mfont__init () < 0)
     goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize font module."));
+  MDEBUG_PRINT_TIME ("INIT", (mdebug__output, " to initialize font module."));
   if (mfont__fontset_init () < 0)
     goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize fontset module."));
+  MDEBUG_PRINT_TIME ("INIT", (mdebug__output, " to initialize fontset module."));
   if (mface__init () < 0)
     goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize face module."));
+  MDEBUG_PRINT_TIME ("INIT", (mdebug__output, " to initialize face module."));
   if (mdraw__init () < 0)
     goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize draw module."));
+  MDEBUG_PRINT_TIME ("INIT", (mdebug__output, " to initialize draw module."));
   if (minput__win_init () < 0)
     goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize input-win module."));
+  MDEBUG_PRINT_TIME ("INIT",
+		     (mdebug__output, " to initialize input-win module."));
   mframe_default = NULL;
 
   register_device_library (Mx, "libm17n-X");
@@ -291,7 +294,8 @@ m17n_init_win (void)
 
  err:
   MDEBUG_POP_TIME ();
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize the m17n GUI module."));
+  MDEBUG_PRINT_TIME ("INIT",
+		     (mdebug__output, " to initialize the m17n GUI module."));
   MDEBUG_POP_TIME ();
 }
 
@@ -307,7 +311,7 @@ m17n_fini_win (void)
 
   MDEBUG_PUSH_TIME ();
   MDEBUG_PUSH_TIME ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize device modules."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize device modules."));
   MPLIST_DO (plist, device_library_list)
     {
       MDeviceLibraryInterface *interface = MPLIST_VAL (plist);
@@ -329,18 +333,19 @@ m17n_fini_win (void)
 #endif	/* not HAVE_FREETYPE */
   M17N_OBJECT_UNREF (device_library_list);
   minput__win_fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize input-gui module."));
+  MDEBUG_PRINT_TIME ("FINI",
+		     (mdebug__output, " to finalize input-gui module."));
   mdraw__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize draw module."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize draw module."));
   mface__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize face module."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize face module."));
   mfont__fontset_fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize fontset module."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize fontset module."));
   mfont__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize font module."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize font module."));
   mframe_default = NULL;
   MDEBUG_POP_TIME ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the gui modules."));
+  MDEBUG_PRINT_TIME ("FINI", (mdebug__output, " to finalize the gui modules."));
   MDEBUG_POP_TIME ();
   m17n_fini_flt ();
   m17n_fini ();
@@ -372,8 +377,8 @@ m17n_fini_win (void)
     These are the symbols to use in a parameter to create a frame.  See
     the function mframe () for details.
 
-    #Mdevice, #Mdisplay, #Mscreen, #Mdrawable, #Mdepth, and #Mcolormap
-    are also keys of a frame property.  */
+    @b Mdevice, @b Mdisplay, @b Mscreen, @b Mdrawable, @b Mdepth, and
+    @b Mcolormap are also keys of a frame property.  */
 
 /***ja
     @name ѿ ե졼ѥ᡼ѥ
@@ -381,8 +386,8 @@ m17n_fini_win (void)
     ե졼ݤΥѥ᡼Ѥ륷ܥ롣ܤϴؿ
     mframe () ȡ
 
-    #Mdevice #Mdisplay #Mscreen #Mdrawable #Mdepth#Mcolormap
-    ϥե졼ץѥƥΥǤ⤢롣  */
+    @b Mdevice @b Mdisplay @b Mscreen @b Mdrawable @b Mdepth
+    @b Mcolormap ϥե졼ץѥƥΥǤ⤢롣  */
 
 /*=*/
 
@@ -425,14 +430,14 @@ MSymbol Mfont_descent;
 
     <ul>
 
-    <li> #Mdevice, the value must be one of #Mx, #Mgd, and #Mnil.
+    <li> @b Mdevice, the value must be one of #Mx, @b Mgd, and #Mnil.
 
     If the value is #Mx, the frame is for X Window System.  The
     argument #MDrawWindow specified together with the frame must be of
     type @c Window.  The frame is both readable and writable, thus all
     GUI functions can be used.
 
-    If the value is #Mgd, the frame is for an image object of GD
+    If the value is @b Mgd, the frame is for an image object of GD
     library.  The argument #MDrawWindow specified together with the
     frame must be of type @c gdImagePtr.  The frame is writable
     only, thus functions minput_XXX can't be used for the frame.
@@ -448,23 +453,23 @@ MSymbol Mfont_descent;
 
     </ul>
 
-    In addition, if the value of the key #Mdevice is #Mx, the
+    In addition, if the value of the key @b Mdevice is #Mx, the
     following keys are recognized.  They are to specify the root
     window and the depth of drawables that can be used with the frame.
 
     <ul>
 
-    <li> #Mdrawable, the value type must be <tt>Drawable</tt>.
+    <li> @b Mdrawable, the value type must be <tt>Drawable</tt>.
 
-    A parameter of key #Mdisplay must also be specified.  The
+    A parameter of key @b Mdisplay must also be specified.  The
     created frame can be used for drawables whose root window and
     depth are the same as those of the specified drawable on the
     specified display.
 
-    When this parameter is specified, the parameter of key #Mscreen
+    When this parameter is specified, the parameter of key @b Mscreen
     is ignored.
 
-    <li> #Mwidget, the value type must be <tt>Widget</tt>.
+    <li> @b Mwidget, the value type must be <tt>Widget</tt>.
 
     The created frame can be used for drawables whose root window and
     depth are the same as those of the specified widget.
@@ -472,34 +477,34 @@ MSymbol Mfont_descent;
     If a parameter of key #Mface is not specified, the default face
     is created from the resources of the widget.
 
-    When this parameter is specified, the parameters of key #Mdisplay,
-    #Mscreen, #Mdrawable, #Mdepth are ignored.
+    When this parameter is specified, the parameters of key @b Mdisplay,
+    @b Mscreen, @b Mdrawable, @b Mdepth are ignored.
 
-    <li> #Mdepth, the value type must be <tt>unsigned</tt>.
+    <li> @b Mdepth, the value type must be <tt>unsigned</tt>.
 
     The created frame can be used for drawables of the specified
     depth.
 
-    <li> #Mscreen, the value type must be <tt>(Screen *)</tt>.
+    <li> @b Mscreen, the value type must be <tt>(Screen *)</tt>.
 
     The created frame can be used for drawables whose root window is
     the same as the root window of the specified screen, and depth is
     the same at the default depth of the screen.
 
-    When this parameter is specified, parameter of key #Mdisplay is
+    When this parameter is specified, parameter of key @b Mdisplay is
     ignored.
 
-    <li> #Mdisplay, the value type must be <tt>(Display *)</tt>.
+    <li> @b Mdisplay, the value type must be <tt>(Display *)</tt>.
 
     The created frame can be used for drawables whose root window is
     the same as the root window for the default screen of the display,
     and depth is the same as the default depth of the screen.
 
-    <li> #Mcolormap, the value type must be <tt>(Colormap)</tt>.
+    <li> @b Mcolormap, the value type must be <tt>(Colormap)</tt>.
 
     The created frame uses the specified colormap.
 
-    <li> #Mfont, the value must be #Mx, #Mfreetype, or #Mxft.
+    <li> @b Mfont, the value must be #Mx, #Mfreetype, or #Mxft.
 
     The created frame uses the specified font backend.  The value #Mx
     instructs to use X core fonts, #Mfreetype to use local fonts
@@ -530,14 +535,14 @@ MSymbol Mfont_descent;
 
     <ul>
 
-    <li> #Mdevice. ͤ #Mx, #Mgd, #Mnil Τ줫ǤʤƤϤʤʤ
+    <li> @b Mdevice. ͤ #Mx, @b Mgd, #Mnil Τ줫ǤʤƤϤʤʤ
 
     ͤ #Mx ʤСե졼 X ɥƥѤǤ롣
     Υե졼ȶ˻ꤵ줿 #MDrawWindow ϡ @c Window
     ǤʤƤϤʤʤե졼ɤ߽񤭤Ȥ˲ǽǤꡢ٤ƤGUI 
     ؿѤǤ롣
 
-    ͤ #Mgd ʤСե졼 GD 
+    ͤ @b Mgd ʤСե졼 GD 
     饤֥Υ᡼֥ѤǤ롣Υե졼ȶ˻ꤵ줿
     #MDrawWindow ϡ @c gdImagePtr ǤʤƤϤʤʤե졼Ͻ񤭽ФѤǤꡢ
     minput_ ǻϤޤ̾δؿϻѤǤʤ
@@ -552,21 +557,21 @@ MSymbol Mfont_descent;
 
     </ul>
 
-    Υ˲ä#Mdevice Υ #Mx 
+    Υ˲ä@b Mdevice Υ #Mx 
     Ǥ˸¤ʲΥǧ롣ʲΥϥ롼ȥɥȡե졼ѤǤ 
     drawable οꤹ롣
 
     <ul>
 
-    <li> #Mdrawable. ͤ <tt>Drawable</tt> ǤʤƤϤʤʤ
+    <li> @b Mdrawable. ͤ <tt>Drawable</tt> ǤʤƤϤʤʤ
 
-     #Mdisplay ĥѥ᡼ꤵƤɬפ롣
+     @b Mdisplay ĥѥ᡼ꤵƤɬפ롣
     줿ե졼ϡꤵ줿ǥץ쥤λꤵ줿 drawable 
     Ʊ롼ȥɥȿ drawable Ѥ롣
 
-    Υѥ᡼ˤϡ#Mscreen 򥭡Ȥѥ᡼̵뤵롣
+    Υѥ᡼ˤϡ@b Mscreen 򥭡Ȥѥ᡼̵뤵롣
 
-    <li> #Mwidget. ͤ <tt>Widget</tt> ǤʤƤϤʤʤ
+    <li> @b Mwidget. ͤ <tt>Widget</tt> ǤʤƤϤʤʤ
 
     줿ե졼ϡꤷåȤƱ롼ȥɥȿ
     drawable Ѥ롣
@@ -574,24 +579,24 @@ MSymbol Mfont_descent;
      #Mface ĥѥ᡼ʤСǥեȤΥեϤ
     åȤΥ꥽롣
 
-    Υѥ᡼ˤϡ#Mdisplay, #Mscreen, #Mdrawable,
-    #Mdepth 򥭡Ȥѥ᡼̵뤵롣
+    Υѥ᡼ˤϡ@b Mdisplay, @b Mscreen, @b Mdrawable,
+    @b Mdepth 򥭡Ȥѥ᡼̵뤵롣
 
-    <li> #Mdepth. ͤ <tt>unsigned</tt>  ǤʤƤϤʤʤ
+    <li> @b Mdepth. ͤ <tt>unsigned</tt>  ǤʤƤϤʤʤ
 
     줿ե졼ϡꤷ drawable Ѥ롣
 
-    <li> #Mscreen. ͤ <tt>(Screen *)</tt> ǤʤƤϤʤʤ
+    <li> @b Mscreen. ͤ <tt>(Screen *)</tt> ǤʤƤϤʤʤ
 
     ե졼ϡꤷ꡼Ʊ롼ȥɥ꡼ΥǥեȤοƱ drawable Ѥ롣
 
-    Υѥ᡼ˤϡ#Mdisplay 򥭡Ȥѥ᡼̵뤵롣
+    Υѥ᡼ˤϡ@b Mdisplay 򥭡Ȥѥ᡼̵뤵롣
 
-    <li> #Mdisplay. ͤ <tt>(Display *)</tt> ǤʤƤϤʤʤ
+    <li> @b Mdisplay. ͤ <tt>(Display *)</tt> ǤʤƤϤʤʤ
 
     줿ե졼ϡꤷǥץ쥤Υǥեȥ꡼Ʊ롼ȥɥƱdrawables Ѥ롣
 
-    <li> #Mcolormap. ͤ <tt>(Colormap)</tt> ǤʤƤϤʤʤ
+    <li> @b Mcolormap. ͤ <tt>(Colormap)</tt> ǤʤƤϤʤʤ
 
     줿ե졼ϡꤷ顼ޥåפѤ롣
 
