@@ -1,5 +1,5 @@
 /* mtext.c -- M-text module.
-   Copyright (C) 2003, 2004, 2005
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -1671,7 +1671,7 @@ mtext ()
     short ñ̤Ǥ롣
 
     $FORMAT  #MTEXT_FORMAT_UTF_32LE  #MTEXT_FORMAT_UTF_32BE ʤС
-    $DATA Ƥ@c unsigned Ǥꡢ$NITEMS  unsigned ñ̤Ǥ롣
+    $DATA Ƥ @c unsigned Ǥꡢ$NITEMS  unsigned ñ̤Ǥ롣
 
     Ƥ줿 M-text ʸѹǤʤ$DATA Ƥ 
     M-text ͭʴ֤ѹƤϤʤʤ
@@ -2376,9 +2376,9 @@ mtext_copy (MText *mt1, int pos, MText *mt2, int from, int to)
 /***ja
     @brief ϰϤʸ˲Ū˼.
 
-    ؿ mtext_del () ϡM-text $MT  $FROM $FROM Τޤˤ $TO
-    $TO ΤϴޤޤʤˤޤǤʸ˲Ū˼Ū $MT Ĺ ($TO @c
-    - $FROM) ̤ळȤˤʤ롣
+    ؿ mtext_del () ϡM-text $MT  $FROM $FROM Τޤˤ
+    $TO $TO ΤϴޤޤʤˤޤǤʸ˲Ū˼Ū
+    $MT Ĺ ($TO @c - $FROM) ̤ळȤˤʤ롣
 
     @return
      mtext_del ()  0 ֤Ǥʤ -1 
@@ -3582,8 +3582,9 @@ mtext_uppercase (MText *mt)
     @brief Dump an M-text.
 
     The mdebug_dump_mtext () function prints the M-text $MT in a human
-    readable way to the stderr.  $INDENT specifies how many columns to
-    indent the lines but the first one.  If $FULLP is zero, this
+    readable way to the stderr or to what specified by the environment
+    variable MDEBUG_OUTPUT_FILE.  $INDENT specifies how many columns
+    to indent the lines but the first one.  If $FULLP is zero, this
     function prints only a character code sequence.  Otherwise, it
     prints the internal byte sequence and text properties as well.
 
@@ -3592,10 +3593,11 @@ mtext_uppercase (MText *mt)
 /***ja
     @brief M-text פ.
 
-    ؿ mdebug_dump_mtext ()  M-text $MT  stderr 
-    ˿ʹ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
-    $FULLP  0 ʤСʸ롣
-    ǤʤСХȥƥȥץѥƥ롣
+    ؿ mdebug_dump_mtext ()  M-text $MT ɸ२顼Ϥ⤷ϴ
+    ѿ MDEBUG_DUMP_FONT ǻꤵ줿ե˿ʹ֤˲ɤʷǰ
+    롣 $INDENT ϣܰʹߤΥǥȤꤹ롣$FULLP  0 ʤ
+    Сʸ롣ǤʤСХȥƥ
+    ȥץѥƥ롣
 
     @return
     δؿ $MT ֤  */
@@ -3607,23 +3609,23 @@ mdebug_dump_mtext (MText *mt, int indent, int fullp)
 
   if (! fullp)
     {
-      fprintf (stderr, "\"");
+      fprintf (mdebug__output, "\"");
       for (i = 0; i < mt->nchars; i++)
 	{
 	  int c = mtext_ref_char (mt, i);
 
 	  if (c == '"' || c == '\\')
-	    fprintf (stderr, "\\%c", c);
+	    fprintf (mdebug__output, "\\%c", c);
 	  else if ((c >= ' ' && c < 127) || c == '\n')
-	    fprintf (stderr, "%c", c);
+	    fprintf (mdebug__output, "%c", c);
 	  else
-	    fprintf (stderr, "\\x%02X", c);
+	    fprintf (mdebug__output, "\\x%02X", c);
 	}
-      fprintf (stderr, "\"");
+      fprintf (mdebug__output, "\"");
       return mt;
     }
 
-  fprintf (stderr,
+  fprintf (mdebug__output,
 	   "(mtext (size %d %d %d) (cache %d %d)",
 	   mt->nchars, mt->nbytes, mt->allocated,
 	   mt->cache_char_pos, mt->cache_byte_pos);
@@ -3636,11 +3638,11 @@ mdebug_dump_mtext (MText *mt, int indent, int fullp)
       memset (prefix, 32, indent);
       prefix[indent] = 0;
 
-      fprintf (stderr, "\n%s (bytes \"", prefix);
+      fprintf (mdebug__output, "\n%s (bytes \"", prefix);
       for (i = 0; i < mt->nbytes; i++)
-	fprintf (stderr, "\\x%02x", mt->data[i]);
-      fprintf (stderr, "\")\n");
-      fprintf (stderr, "%s (chars \"", prefix);
+	fprintf (mdebug__output, "\\x%02x", mt->data[i]);
+      fprintf (mdebug__output, "\")\n");
+      fprintf (mdebug__output, "%s (chars \"", prefix);
       p = mt->data;
       for (i = 0; i < mt->nchars; i++)
 	{
@@ -3648,21 +3650,21 @@ mdebug_dump_mtext (MText *mt, int indent, int fullp)
 	  int c = STRING_CHAR_AND_BYTES (p, len);
 
 	  if (c == '"' || c == '\\')
-	    fprintf (stderr, "\\%c", c);
+	    fprintf (mdebug__output, "\\%c", c);
 	  else if (c >= ' ' && c < 127)
-	    fputc (c, stderr);
+	    fputc (c, mdebug__output);
 	  else
-	    fprintf (stderr, "\\x%X", c);
+	    fprintf (mdebug__output, "\\x%X", c);
 	  p += len;
 	}
-      fprintf (stderr, "\")");
+      fprintf (mdebug__output, "\")");
       if (mt->plist)
 	{
-	  fprintf (stderr, "\n%s ", prefix);
+	  fprintf (mdebug__output, "\n%s ", prefix);
 	  dump_textplist (mt->plist, indent + 1);
 	}
     }
-  fprintf (stderr, ")");
+  fprintf (mdebug__output, ")");
   return mt;
 }
 
